/*

             **************************************************
             ****             VEGA - ArgPar V1.1           ****
             ****              Arguments parser            ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************


*/

#include <stdio.h>
#include <string.h>

#include "globdef.h"
#include "globvar.h"
#include "formats.h"

#define  MAXOPT         MAXTRJATM
#define  MAXSUBOPT      3

/**** Local variables ****/

static char             Str[50];

/**** Local prototypes ****/

static BOOL AtmMatch(ULONG *, ULONG, UWORD);



/**** Copy the atom number ****/

static BOOL AtmMatch(ULONG *Dest, ULONG Src, UWORD Idesc)
{
  BOOL          Ret = TRUE;

  if (*Dest) {
    sprintf(Str, "The atom description %d has multiple matches", Idesc + 1);
    Ret = Errore(Str);
  } else *Dest = Src;

  return Ret;
}


/**** Translate atom description into atom number ****/

BOOL AtmTrans(ATOMO *InizAtm, ULONG TotAtm, register TRJPAR *TrjPar)
{
  ATMDESC               Tmp;
  register ATOMO         *Atm;
  register ATMDESC      *Desc;
  UWORD                 j, k;

  BOOL                  Ret = TRUE;

  if (TrjPar -> Com <= TRJMS_TORSION) {
    for(k = 0; (Ret) && (k < TrjPar -> Num); ++k) {
      Desc = &TrjPar -> Atm[k];
      if (!Desc -> Num) {
        for(Atm = InizAtm; (Ret) && (Atm); Atm = Atm -> Ptr) {
          if (Atm -> Name.L == Desc -> Name.L) {
            if (Desc -> ResName.L) {
              if (Atm -> ResName.L == Desc -> ResName.L) {
                if (Desc -> ResSeq.L) {
                  if (Atm -> ResSeq.L == Desc -> ResSeq.L)
                    Ret = AtmMatch(&Desc -> Num, Atm -> Num, k);
                } else Ret = AtmMatch(&Desc -> Num, Atm -> Num, k);
              }
            } else Ret = AtmMatch(&Desc -> Num, Atm -> Num, k);
          }
        } /* End of ATOMO for*/
        if (!Desc -> Num) {
          sprintf(Str, "The atom description %d don't match the input file",
                  k + 1);
          Ret = Errore(Str);
        }
      } else if (Desc -> Num > TotAtm) {
        sprintf(Str, "The atom descrpition %d exceeds the total number", k + 1);
        Ret = Errore(Str);
      }
      for(j = 0; (Ret) && (j < TrjPar -> Num); ++j)
        if ((j != k) && (Desc -> Num == TrjPar -> Atm[j].Num)) {
          sprintf(Str, "Duplicate description for atom %d and %d",
                  k + 1, j + 1);
          Ret = Errore(Str);
        }

    } /* End of PAR for */

    /**** Sort atom descriptors ****/

    if (Ret) {
      for(j = 0; j < TrjPar -> Num; ++j) {
        Desc = &TrjPar -> Atm[j];
        for(k = j; k < TrjPar -> Num; ++k) {
          if (Desc -> Num > TrjPar -> Atm[k].Num)
            Desc = &TrjPar -> Atm[k];
        } /* End of for loop */
        if (Desc -> Num != TrjPar -> Atm[j].Num) {
          memcpy(&Tmp, &TrjPar -> Atm[j], sizeof(ATMDESC));
          memcpy(&TrjPar -> Atm[j], Desc, sizeof(ATMDESC));
          memcpy(Desc, &Tmp, sizeof(ATMDESC));
        }
      } /* End of for loop */
    }
  }

  return Ret;
}


/**** Analysis command parser ****/

BOOL TrjParser(TRJPAR *TrjPar)
{
  ATMDESC       *Desc;
  char          *Arg, *PrecArg, *SubOpt[MAXSUBOPT];
  UWORD         j, k, TotSubOpt;

  BOOL          Ret        = TRUE;
  const char    *ModeStr[] = { "DISTANCE", "ANGLE", "TORSION", "PLANEANG",
                               "VLOGP", "DIPOLE", "SURFACE", "SURFDIA",
                               "VOLUME", "VOLDIA", NULL };
  const BYTE    ModeArgs[] = { 2, 3, 4, 6, 0, 0, -1, -1, 0, 0 };

  if (TrjPar -> Num >= 1) {
    for(j = 0; (ModeStr[j]) && (strcasecmp(TrjPar -> Arg[0], ModeStr[j])); ++j);
    if (ModeStr[j]) {
      TrjPar -> Com = j;
      --TrjPar -> Num;
      if ((ModeArgs[TrjPar -> Com] != (BYTE)-1) ||
          ((ModeArgs[TrjPar -> Com] == (BYTE)-1) && (TrjPar -> Num))) {
        if ((ModeArgs[TrjPar -> Com] == (BYTE)-1) ||
            (TrjPar -> Num == ModeArgs[TrjPar -> Com])) {
          ++TrjPar -> Arg;
          for(k = 0; (Ret) && (k < TrjPar -> Num); ++k) {
            Desc              = &TrjPar -> Atm[k];
            Desc -> Num       = 0;
            Desc -> Name.L    = 0;
            Desc -> ResName.L = 0;
            Desc -> ResSeq.L  = 0;
            Arg               = TrjPar -> Arg[k];
            if (!SubArg(&Arg, SUBA_POSINT, FALSE)) {
              PrecArg   = Arg;
              SubOpt[0] = Arg;
              TotSubOpt = 1;

              /**** Parse the atom description ****/
              /**** Format: ATOM:RESN:RNUM     ****/

              while((Ret) && (*Arg)) {
                if (*Arg == ':') *Arg = 0;
                else if (!*PrecArg) {
                  if (TotSubOpt < MAXSUBOPT) SubOpt[TotSubOpt++] = Arg;
                  else Ret = Errore("Incorrect atom description");
                }
                PrecArg = Arg++;
              } /* End of while */

              for(j = 0;(Ret) && (j < TotSubOpt); ++j) {
                if (strlen(SubOpt[j]) <= sizeof(QCHAR)) {
                  switch(j) {
                  case 0:         /* Atom name      */
                    ResConv(&TrjPar -> Atm[k].Name, SubOpt[j]);
                    break;
                  case 1:         /* Residue name   */
                    ResConv(&TrjPar -> Atm[k].ResName, SubOpt[j]);
                    break;
                  case 2:         /* Residue number */
                    ResConv(&TrjPar -> Atm[k].ResSeq, SubOpt[j]);
                    break;
                  }
                } else Ret = Errore("Atom description too long");
              } /* End of for loop */
            } else {
              sscanf(TrjPar -> Arg[k], "%d", &TrjPar -> Atm[k].Num);
              if (!TrjPar -> Atm[k].Num) Ret = Errore("Illegal atom number");
            }
          } /* End of for loop */
        } else Ret = Errore("Illegal number of atoms in mesure");
      }
    } else Ret = Errore("Unknown mesure mode");
  } else Ret = Errore("Too few atoms to mesure");


  return Ret;
}

