/*

             **************************************************
             ****             VEGA - Charge V1.1           ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************


*/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "globdef.h"
#include "globvar.h"

#define  ATOMO_TYPES	"UNIV.tem"


static BOOL ChargeTem(char *, ATOMO *, ULONG);


/**** Dipole calculation ****/

float CalcDipole(register ATOMO *Atm)
{
  float		Val;

  float		Xp = 0.0;
  float		Yp = 0.0;
  float		Zp = 0.0;
  float		Xn = 0.0;
  float		Yn = 0.0;
  float		Zn = 0.0;
  float		Cp = 0.0;
  float		Cn = 0.0;

  while(Atm) {
    Val = fabs(Atm -> Charge);
    if (Atm -> Charge > 0) {
      Xp += Atm -> x * Val;
      Yp += Atm -> y * Val;
      Zp += Atm -> z * Val;
      Cp += Val;
    } else {
      Xn += Atm -> x * Val;
      Yn += Atm -> y * Val;
      Zn += Atm -> z * Val;
      Cn += Val;
    }
    Atm = Atm -> Ptr;
  }
  if (Cp) {
    Xp /= Cp;
    Yp /= Cp;
    Zp /= Cp;
  }
  if (Cn) {
    Xn /= Cn;
    Yn /= Cn;
    Zn /= Cn;
  }

  return (SQR(Quad(Xp - Xn) + Quad(Yp - Yn) + Quad(Zp - Zn)) * DEBYE_CONST);
}


/**** Calculate a single point MEP ****/

float CalcSrfMep(register ATOMO *Atm, float X, float Y, float Z)
{
  register float         Dist;
  register float         Mep = 0;

  do {
    Dist = Quad(X - Atm -> x) + Quad(Y - Atm -> y) + Quad(Z - Atm -> z);
    Mep += Atm -> Charge / SQR(Dist);
    Atm  = Atm -> Ptr;
  } while(Atm);

  return Mep;
}


/**** Select charge routine ****/

BOOL Charge(char *Template, UWORD Type, ATOMO *InizAtm, ULONG TotAtm)
{
  BOOL			Ret;
  char			FFTemp[256];

  PrintProg("* Assigning charges");

  switch(Type) {
  case CHARGE_FRAG:
    Ret = ChargeTem(Template, InizAtm, TotAtm);
    break;
  case CHARGE_GAST:
    if (Ret = GetPrgPath(FFTemp)) {
      strcat(FFTemp, ATOMO_TYPES);
      if (Ret = AssignFF(FFTemp, InizAtm, TotAtm, FALSE))
        Ret = Gasteiger(Template, InizAtm, TotAtm);
    }
    break;
  }

  return Ret;
}


/**** Assign charges trough library of fragments ****/

static BOOL ChargeTem(char *Template, ATOMO *InizAtm, ULONG TotAtm)
{
  char			Buf[256], Com[16], ExtRes[12], Res[12];
  FILE			*FH;
  float			CharPar;
  QCHAR			FastRes, Iiub;
  register ATOMO		*Atm;
  ULONG			k;

  static const char	*Commands[] = { "RESNAME",
                                        "CHARGE",
                                        "DESC",
                                        "ENDDESC",
                                        "TITLE",
                                        ""
                                      };

  BOOL			BegDesc = FALSE;
  BOOL			Ret     = TRUE;
  ULONG			Line    = 1;


  FindRes(InizAtm, TotAtm);

  if (FH = fopen(Template, "r")) {
    fgets(Buf, 255, FH);

    /**** Clean the charges ****/

    for(Atm = InizAtm; Atm; Atm = Atm -> Ptr) Atm -> Charge = 10;

    while(fgets(Buf, 255, FH)) {
      if ((*Buf != '\n') && (*Buf != ';')) {
        if (*Buf == '#') {
          sscanf(Buf + 1, "%s", Com);
          for(k = 0;(*Commands[k]) && (strcasecmp(Com, Commands[k])); ++k);
          if (!*Commands[k]) {
            ChargeErr("Unknown command", Buf, Line);
            break;
          }

          if (k == 2) BegDesc = TRUE;		/* Start the atom description  */
          else if (k == 3) BegDesc = FALSE;	/* Stop the atom description   */
            else if (!BegDesc) {
              switch(k) {

              case 0:	/* Residue name */
      	      	ExtRes[1] = 0;
      	      	ExtRes[2] = 0;
      	      	ExtRes[3] = 0;
              	FastRes.L = (LONG)NULL;
              	sscanf(Buf, "%*s %s %s", Res, ExtRes);
              	strncpy(FastRes.C, ExtRes, 4);
              	break;

              case 1:	/* Total charge */
              	break;

              case 4:	/* Template name */
      	      	if (!GlobSwitch[4]) printf("  Database name: %s", Buf + 7);
              	break;
              }
            } else {
              ChargeErr("Illegal command in atom description", Buf, Line);
              Ret = FALSE;
              break;
            }
        } else if (BegDesc) {
          Com[1] = 0;
          Com[2] = 0;
          Com[3] = 0;
          Iiub.L = (LONG)NULL;
          sscanf(Buf, "%s", Com);
          strncpy(Iiub.C, Com, 4);
          sscanf(Buf + 61, "%f", &CharPar);
          for(Atm = InizAtm; Atm; Atm = Atm -> Ptr)
            if ((Atm -> ResName.L == FastRes.L) && (Atm -> Name.L == Iiub.L))
              Atm -> Charge = CharPar;
        }
      }
      ++Line;
    }
    fclose(FH);
  }

  /**** Check the charges ****/

  if (Ret) {
    BegDesc = TRUE;
    CharPar = 0;
    for(Atm = InizAtm; Atm; Atm = Atm -> Ptr)
      if (Atm -> Charge == 10) {
        Atm -> Charge = 0;
        PrintAtmErr("  Unknown charge", Atm);
        BegDesc = FALSE;
      } else CharPar += Atm -> Charge;
    if ((BegDesc) && (!GlobSwitch[4])) printf("  Total charge: %.4f\n", CharPar);
  }

  return Ret;
}


/**** Shows an error ****/

void ChargeErr(char *ErrMsg, char *Str, ULONG Line)
{
  fprintf(stderr, "\nLine #%d -> \"%s\"\n%s\n",
          Line, ErrMsg, Str);
}


/**** Find interesting residues ****/

LONG FindRes(ATOMO *InizAtm, ULONG TotAtm)
{
  char			*Ptr;
  QCHAR			Res, ResName;
  register ATOMO		*Atm;
  register UWORD	k;
  WORD			Chk;

  ATOMO			*InizRes  = InizAtm;
  char			*CysCar   = "CSH";
  LONG			TotCharge = 0;

  FFDESC		NTerm[]  = {{"N", 3, 0, 0, 0},
                                    {"H", 1, 0, 0, 1},
                                    {"H", 1, 0, 0, 1},
                                    {"C", 4, 0, 0, 1},
                                    {"H", 1, 0, 0, 2},
                                    {"C", 3, 0, 0, 2},
                                    {"O", 1, 0, 0, 3},
                                    {"N", 3, 0, 0, 3}
                                   };

  FFDESC		NTermC[] = {{"N", 4, 0, 0, 0},
                                    {"H", 1, 0, 0, 1},
                                    {"H", 1, 0, 0, 1},
                                    {"H", 1, 0, 0, 1},
                                    {"C", 4, 0, 0, 1},
                                    {"H", 1, 0, 0, 2},
                                    {"C", 3, 0, 0, 2},
                                    {"O", 1, 0, 0, 3},
                                    {"N", 3, 0, 0, 3}
                                   };

  FFDESC		Arg[]    = {{"C", 3, 0, 0, 0},
			            {"N", 3, 0, 0, 1},
			            {"N", 3, 0, 0, 1},
			            {"N", 3, 0, 0, 1},
			           };

  FFDESC		AspGlu[] = {{"C", 3, 0, 0, 0},
			            {"O", 1, 0, 0, 1},
			            {"O", 1, 0, 0, 1},
			            {"C", 4, 0, 0, 1},
			            {"H", 1, 0, 0, 2},
			            {"H", 1, 0, 0, 2}
			           };

  FFDESC		CTerm[]  = {{"C", 3, 0, 0, 0},
			            {"O", 1, 0, 0, 1},
			            {"O", 2, 0, 0, 1},
			            {"H", 1, 0, 0, 2},
			            {"C", 4, 0, 0, 1},
			            {"H", 1, 0, 0, 2},
			            {"N", 3, 0, 0, 2},
			            {"C", 3, 0, 0, 2}
			           };

  FFDESC		CTermC[] = {{"C", 3, 0, 0, 0},
			            {"O", 1, 0, 0, 1},
			            {"O", 1, 0, 0, 1},
			            {"C", 4, 0, 0, 1},
			            {"H", 1, 0, 0, 2},
			            {"N", 3, 0, 0, 2},
			            {"C", 3, 0, 0, 2}
			           };

  FFDESC		Lys[]    = {{"N", 4, 0, 0, 0},
			            {"H", 1, 0, 0, 1},
			            {"H", 1, 0, 0, 1},
			            {"H", 1, 0, 0, 1},
			            {"C", 4, 0, 0, 1},
			            {"H", 1, 0, 0, 2},
			            {"H", 1, 0, 0, 2},
			           };

  if (MatchMtx = (UBYTE *)Alloca(TotAtm)) {

    PrintProg("* Searching for residues");

    for(Atm = InizAtm; Atm; Atm = Atm -> Ptr) {
      ResName.L = Atm -> ResName.L;
      if ((InizRes -> ResSeq.L != Atm -> ResSeq.L) ||
          (InizRes -> ResName.L != ResName.L)) InizRes = Atm;

      /**** Find charged GLU and ASP ****/

      if ((ResName.L == *(LONG *)"GLU") || (ResName.L == *(LONG *)"ASP")) {
        if (ChkFFAtm(Atm, &AspGlu[0])) {
          memset(MatchMtx, 0, TotAtm);
          if (MatchAtm(Atm, &AspGlu[1], &AspGlu[5])) {
            --TotCharge;
            ResName.C[3] = '-';
/*
            printf("%.2s %.2s %4.4s\n", Atm -> Conn[0] -> Elem.C,
              Atm -> Conn[1] -> Elem.C, InizRes -> ResSeq.C);
*/
            Atm = RenameRes(InizRes, ResName.L);
          }
        }
      } else if ((ResName.L == *(LONG *)"LYS")) {

        /**** Find charged LYS ****/

      	if (ChkFFAtm(Atm, &Lys[0])) {
          memset(MatchMtx, 0, TotAtm);
          if (MatchAtm(Atm, &Lys[1], &Lys[6])) {
            ++TotCharge;
            Atm = RenameRes(InizRes, *(LONG *)"LYS+");
          }
      	}
      } else if ((ResName.L == *(LONG *)"ARG")) {

        /**** Find charged ARG ****/

      	if (ChkFFAtm(Atm, &Arg[0])) {
          memset(MatchMtx, 0, TotAtm);
          if (MatchAtm(Atm, &Arg[1], &Arg[3])) {
            ++TotCharge;
            Atm = RenameRes(InizRes, *(LONG *)"ARG+");
          }
      	}
      } else

      /**** Find CYS and check sulfide bridges ****/

      	if ((ResName.L == *(LONG *)"CYS")  &&  (Atm -> Elem.S == 0x5300)) {
      	  if (Atm -> NSost == 2) {
      	    Chk = 0;
      	    for(k = 0; k < 2; ++k)
      	      for(Ptr = CysCar; *Ptr; ++Ptr)
      	        if ((Atm -> Conn[k] -> Elem.C[0] == *Ptr) &&
      	            (!Atm -> Conn[k] -> Elem.C[1])) {
      	          ++Chk;
      	          break;
      	        }
      	    if (Chk != 2) --TotCharge;
      	    if ((Atm -> Conn[0] -> Elem.S == 0x4800) || (Atm -> Conn[1] -> Elem.S == 0x4800))
      	      Atm = RenameRes(InizRes, *(LONG *)"CYH");

        } else if (Atm ->  NSost == 1) --TotCharge;
      }
    }

    /**** Find the chain terms ****/

    InizRes = InizAtm;
    for(Atm = InizAtm; Atm; Atm = Atm -> Ptr) {
      ResName.L = Atm -> ResName.L;
      if ((InizRes -> ResSeq.L != Atm -> ResSeq.L) ||
          (InizRes -> ResName.L != ResName.L)) InizRes = Atm;
      for(k = 0; *AAResTab[k].C; ++k) {
        if (AAResTab[k].L == Atm -> ResName.L) {
          if (ChkFFAtm(Atm, &NTermC[0])) {
            memset(MatchMtx, 0, TotAtm);
            if (MatchAtm(Atm, &NTermC[1], &NTermC[8])) {
              Res.L = ResName.L;
              SetTypeRes(&Res, 'N');
              Atm = RenameRes(InizRes, Res.L);
              ++TotCharge;
            }
          } else if (ChkFFAtm(Atm, &NTerm[0])) {
            memset(MatchMtx, 0, TotAtm);
            if (MatchAtm(Atm, &NTerm[1], &NTerm[7])) {
              Res.L = ResName.L;
              SetTypeRes(&Res, 'n');
              Atm = RenameRes(InizRes, Res.L);
            }
          } else if (ChkFFAtm(Atm, &CTermC[0])) {
            memset(MatchMtx, 0, TotAtm);
            if (MatchAtm(Atm, &CTermC[1], &CTermC[6])) {
              Res.L = ResName.L;
              SetTypeRes(&Res, 'C');
              Atm = RenameRes(InizRes, Res.L);
      	      --TotCharge;
            } else if (MatchAtm(Atm, &CTerm[1], &CTerm[7])) {
              Res.L = ResName.L;
              SetTypeRes(&Res, 'c');
              Atm = RenameRes(InizRes, Res.L);
            }
          }
          break;
        }
      }
    }

    FREE(MatchMtx);
    RenameWat(InizAtm);
  }

  return TotCharge;
}


