/*

             ************************************************
             ****         VEGA - Force Field V1.2        ****
             **** Copyright 1996-98, Alessandro Pedretti ****
             ************************************************


*/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "globdef.h"
#include "globvar.h"

#define  FFDESC_LEN		sizeof(FFDESC)
#define  MAXFFBOND		4
#define  MAXFFSCORE		5
#define  MAXRING		6
#define  MAXSUBLEVELS           256
#define  ORDMTX_LEN		(sizeof(OrdMtxL) / sizeof(LONG))
#define  TOTHUCK		(sizeof(PiMtr) / sizeof(HUCK))


/**** Assign the force field ****/

BOOL AssignFF(char *FFFile, ATOMO *InizAtm, ULONG TotAtomi, BOOL Quiet)
{
  char			Buf[256];
  FFDESC		*FFArgs;
  FFPAR			ParFF;
  QCHAR			FFType;
  register ATOMO		*Atm;
  UBYTE			SubCount;
  ULONG			k;
  UWORD			j;

  FFDESC		*InizFFArgs = NULL;
  BOOL			Ret         = TRUE;
  ULONG			PrecTotArgs = (ULONG)NULL;


  if (MatchMtx = (UBYTE *)Alloca(TotAtomi)) {
    if (InizFFArgs = (FFDESC *)Alloca(MAXSUBLEVELS * FFDESC_LEN)) {
      if (!Quiet) PrintProg("* Fixing atomic potentials");

      /**** Clean the force field ****/

      Atm  = InizAtm;
      while(Atm) {
        Atm -> Pot.L  = (ULONG)NULL;
        *Atm -> Pot.C = '?';
        Atm = Atm -> Ptr;
      }

      ParFF.Line = 2;
      if (ParFF.FFIN = fopen(FFFile, "r")) {
        fgets(Buf, 256, ParFF.FFIN);
        while((Ret) && (fgets(Buf, 256, ParFF.FFIN))) {
          if ((*Buf != ';') && (*Buf != '\n')) {

            /**** Count the number of arguments ****/

            j             = 0;
            k             = 0;
            ParFF.TotArgs = (ULONG)NULL;
            ParFF.Beg     = NULL;
            FFArgs        = InizFFArgs;
            for(ParFF.CPtr = Buf;(*ParFF.CPtr && (*ParFF.CPtr != ';'));++ParFF.CPtr) {
              if (!ParFF.Beg) {
                if ((*ParFF.CPtr != ' ') && (*ParFF.CPtr != '\t') && (*ParFF.CPtr != '(') && (*ParFF.CPtr != ')')) ParFF.Beg = ParFF.CPtr;
              } else switch (*ParFF.CPtr) {
              case '(':
              case ')':
              case ' ':
              case '\n':
              case '\t':
                ++ParFF.TotArgs;
                ParFF.Beg = NULL;
              };
              switch (*ParFF.CPtr) {
              case '(':
                ++j;
                ++k;
                break;
              case ')':
                --k;
                break;
              }
            }
            if (ParFF.TotArgs < 2) {
              ParFF.Beg = NULL;
              FFError(&ParFF, "Illegal number of arguments");
              Ret = FALSE;
              break;
            }
            if (k) {
              ParFF.Beg = NULL;
              FFError(&ParFF, "Unbalenced parenthesis");
              Ret = FALSE;
              break;
            }
            if (j > MAXSUBLEVELS) {
              ParFF.Beg = NULL;
              FFError(&ParFF, "Sublevels overflow");
              Ret = FALSE;
              break;
            }

            /**** Read the arguments ****/

            --ParFF.TotArgs;
            ParFF.Args = (ULONG)NULL;
            ParFF.Beg  = NULL;
            SubCount   = 0;

            for(ParFF.CPtr = Buf;((Ret) && (*ParFF.CPtr) && (*ParFF.CPtr != ';'));++ParFF.CPtr) {
              if (*ParFF.CPtr == '(') ++SubCount;
              if (!ParFF.Beg) {
	        if ((*ParFF.CPtr != ' ') && (*ParFF.CPtr != '\t') &&
	            (*ParFF.CPtr != '(') && (*ParFF.CPtr != ')')) ParFF.Beg = ParFF.CPtr;
              } else switch (*ParFF.CPtr) {
              case '(':
              case ')':
              case ' ':
              case '\n':
              case '\t':
                if (ParFF.Args) FFArgs -> NSub = SubCount;
                if (*ParFF.CPtr == ')') --SubCount;
                *ParFF.CPtr = 0;

                switch (ParFF.Args) {

                case 0:	/* Force field atom name */

                  if ((ParFF.CPtr - ParFF.Beg) > 5) {
                    FFError(&ParFF, "Atom type too long");
                    Ret = FALSE;
                    break;
                  }
/*
                  memcpy(&FFType.L, ParFF.Beg, 4);
*/
                  FFType.L = 0;
                  for(k = 0; (k < 4) && (ParFF.Beg[k] != ' ') &&
                      (ParFF.Beg[k] != '\t') && (ParFF.Beg[k] != '\n'); ++k)
                    FFType.C[k] = ParFF.Beg[k];


                  break;

                case 1:	/* Atom description */

                  if ((ParFF.CPtr - ParFF.Beg) > 6) {
                    FFError(&ParFF, "Incorrect atom description");
                    Ret = FALSE;
                    break;
                  }
                  Ret = ToDesc(&ParFF, InizFFArgs, ParFF.Beg);
                  FFArgs = InizFFArgs + 1;
                  break;

                default: /* Connected atom description */

                  if ((ParFF.CPtr - ParFF.Beg) > 7) {
                    FFError(&ParFF, "Incorrect subatom description");
                    Ret = FALSE;
                    break;
                  }
                  Ret = ToDesc(&ParFF, FFArgs, ParFF.Beg);
                  ++FFArgs;
                }
                ++ParFF.Args;
                ParFF.Beg = NULL;
              };
            }

            if (Ret) {
              Atm  = InizAtm;
              while(Atm) {
                if ((*Atm -> Pot.C == '?') && (ChkFFAtm(Atm, InizFFArgs))) {
                  if (ParFF.TotArgs > 1) {
                    memset(MatchMtx, 0, TotAtomi);
                    if (MatchAtm(Atm, InizFFArgs + 1, InizFFArgs + ParFF.TotArgs - 1)){
/*
                      printf("%4.4s -> %d %4.4s %2.2s %3.3s_%-4.4s\n",
                             FFType.C, Atm -> Num, Atm -> Name.C, Atm -> Elem.C,
                        	   Atm -> ResName.C, Atm -> ResSeq.C);
*/
                      Atm -> Pot.L = FFType.L;
                    }
                  } else Atm -> Pot.L = FFType.L;
                }
                Atm = Atm -> Ptr;
              }
            }
          }
          ++ParFF.Line;
        }
        fclose(ParFF.FFIN);
      } else {
        sprintf(Buf, "\"%s\" not found", FFFile);
        Ret = Errore(Buf);
      }
      FREE(InizFFArgs);
    }
    FREE(MatchMtx);

    /**** Check if the force field is correctly assigned ****/

    if (Ret) {
      Atm  = InizAtm;
      while(Atm) {
      	if (*Atm -> Pot.C == '?')
      	  PrintAtmErr("  Can't assign potential to", Atm);
      	Atm = Atm -> Ptr;
      }
    }
  } else Ret = FALSE;

  return Ret;
}


/**** Match the atom with force field template ****/

UWORD MatchAtm(register ATOMO *Atm, FFDESC *FFStart, FFDESC *FFEnd)
{
  ATOMO			*TmpAtm;
  BOOL			Match;
  register FFDESC	*FFArg, *PrecFFArg;
  ULONG			TmpNum;
  UWORD			k;

  BOOL			Flag = FALSE;
  UWORD			Ret  = 0;

  for(FFArg = FFStart;FFArg <= FFEnd; ++FFArg) {
    for(k = 0;k < Atm -> NSost; ++k) {
      TmpAtm = Atm -> Conn[k];
      TmpNum = TmpAtm -> Num - 1;
      if ((!MatchMtx[TmpNum]) && (ChkFFAtm(TmpAtm, FFArg))) {
        PrecFFArg = FFArg;
        if ((FFArg != FFEnd) && (FFArg -> NSub < (FFArg + 1) -> NSub)) {
          Match  = TRUE;
          Ret    = MatchAtm(TmpAtm, FFArg + 1, FFEnd);
          if (!Ret) return 0;
          FFArg += Ret;
          Flag   = TRUE;
        } else {
          ++Ret;
          Match = FALSE;
          Flag  = TRUE;
        }
        break;
      }
    }

    if (Flag) {
      MatchMtx[TmpNum] = TRUE;
/*
      printf("%d %.4s %2.2s %d\n",
             TmpAtm -> Num, TmpAtm -> ResName.C, TmpAtm -> Name.C, PrecFFArg -> NSub);
*/
      if (FFArg != FFEnd) {
        if (Match) {
          if (((FFArg + 1) <= FFEnd) && (PrecFFArg -> NSub == (FFArg + 1) -> NSub))
            Flag = FALSE;
        } else {
          if (PrecFFArg -> NSub == (PrecFFArg + 1) -> NSub) Flag = FALSE;
          if (PrecFFArg -> NSub > (PrecFFArg + 1) -> NSub) break;
        }
/*
        if ((Match) && ((FFArg + 2) <= FFEnd) &&
          ((FFArg + 1) -> NSub == (FFArg + 2) -> NSub)) Flag = TRUE;
        else {
          if (PrecFFArg -> NSub == (PrecFFArg + 1) -> NSub) Flag = FALSE;
          if (PrecFFArg -> NSub > (PrecFFArg + 1) -> NSub) break;
        }
*/
      }
    } else {
      Ret = 0;
      break;
    }
  }

  return Ret;
}


/**** Check the atom matching ****/

BOOL ChkFFAtm(register ATOMO *Atm, register FFDESC *FFArg)
{
  BOOL		Ret = FALSE;


#ifndef BIG_ENDIAN
  if ((Atm -> Elem.S == FFArg -> Atom.S) || (*FFArg -> Atom.C == 'X') ||
      ((*FFArg -> Atom.C == '#') && (Atm -> Elem.S != 0x4800)) ||
      ((*FFArg -> Atom.C == '$') && (Atm -> Elem.S != 0x4800) && (Atm -> Elem.S != 0x4300))) {
#else
  if ((Atm -> Elem.S == FFArg -> Atom.S) || (*FFArg -> Atom.C == 'X') ||
      ((*FFArg -> Atom.C == '#') && (Atm -> Elem.S != 0x0048)) ||
      ((*FFArg -> Atom.C == '$') && (Atm -> Elem.S != 0x0048) && (Atm -> Elem.S != 0x0043))) {
#endif

    if ((Atm -> NSost == FFArg -> NSost) || (FFArg -> NSost == 9)) {
      if (FFArg -> Ring) {
        if ((FFArg -> Ring == 9) && (Atm -> Ring)) Ret = TRUE;
        else if (abs(Atm -> Ring) == FFArg -> Ring) Ret = TRUE;
        if ((Ret) && (FFArg -> Arom) && (Atm -> Ring > 0)) Ret = FALSE;
      } else Ret = TRUE;
    }
  }

  return Ret;
}


/**** Show an error in force field file ****/

void FFError(FFPAR *ParFF, char *Str)
{
  char		Buf[256];

  if (ParFF -> Beg)
    sprintf(Buf, "at line #%d -> \"%s\"\n%s",
            ParFF -> Line, ParFF -> Beg, Str);
  else
    sprintf(Buf, "at line #%d of template file\n%s", ParFF -> Line, Str);
  Errore(Buf);
}

/**** Find the rings ****/

void FindRing(ATOMO *InizAtm)
{
  ATOMO			*AtmMtr[MAXRING];
  BYTE			NSost0, NSost1, NSost2;
  BYTE			NSost3, NSost4, NSost5;
  BYTE			k, l, N3, Ring, TotPi;
  register ATOMO		*Atm, *Temp;
  WORD			j;

  static HUCK PiMtr[] = {{"C",3,1},	/* Attenzione all'ordine nella matrice */
			 {"N",3,2},
			 {"N",2,1},
			 {"O",2,1},
			 {"S",2,1}};

  PrintProg("* Searching for rings");

  Atm = InizAtm;
  while(Atm) {
    if (Atm -> NSost > 1) {
      Ring    = 0;
      *AtmMtr = Atm;
      for(NSost0 = 0;NSost0 < AtmMtr[0] -> NSost;++NSost0) {
        Temp = AtmMtr[0] -> Conn[NSost0];
        if (Temp -> NSost > 1) {
          AtmMtr[1] = Temp;
          for(NSost1 = 0;NSost1 < AtmMtr[1] -> NSost;++NSost1) {
            Temp = AtmMtr[1] -> Conn[NSost1];
            if ((Temp -> NSost > 1) && (Temp != AtmMtr[1])) {
              AtmMtr[2] = Temp;
              for(NSost2 = 0;NSost2 < AtmMtr[2] -> NSost;++NSost2) {
                Temp = AtmMtr[2] -> Conn[NSost2];
                if ((Temp -> NSost > 1) && (Temp != AtmMtr[1])) {
                  AtmMtr[3] = Temp;
                  if (Temp == Atm) {
                    Ring = 3;
                    goto Esci;
                  }
                  for(NSost3 = 0;NSost3 < AtmMtr[3] -> NSost;++NSost3) {
                    Temp = AtmMtr[3] -> Conn[NSost3];
                    if ((Temp -> NSost > 1) && (Temp != AtmMtr[2])) {
                      AtmMtr[4] = Temp;
                      if (Temp == Atm) {
                        Ring = 4;
                        goto Esci;
                      }
                      for(NSost4 = 0;NSost4 < AtmMtr[4] -> NSost;++NSost4) {
        		Temp = AtmMtr[4] -> Conn[NSost4];
                        if ((Temp -> NSost > 1) && (Temp != AtmMtr[3])) {
                          AtmMtr[5] = Temp;
                          if (Temp == Atm) {
                            Ring = 5;
                            goto Esci;
                          }
                          for(NSost5 = 0;NSost5 < AtmMtr[5] -> NSost;++NSost5) {
                            if (AtmMtr[5] -> Conn[NSost5] == Atm) {
                              Ring = 6;
                              goto Esci;
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
	  }
	}
      }
      Esci:
      if (Ring) {
        N3    = 0;
        TotPi = 0;
        for(k=0; k < Ring;++k) {
           l = 0;
           for(j = 0xf;(AtmMtr[k] -> Ring & j) && (l < 16);j <<= 4) l += 4;
           for(j=0; j < TOTHUCK;++j)
             if ((PiMtr[j].Atom.S == AtmMtr[k] -> Elem.S) && (PiMtr[j].NSost == AtmMtr[k] -> NSost)) {
               if (j == 1) ++N3;		/* Attenzione all'ordine nella matrice */
               TotPi += PiMtr[j].Npi;
               break;
             };
	   AtmMtr[k] -> Ring += Ring << l;

        }

	if ((TotPi == 6) || ((TotPi == 7) && (N3 ==2))) {
/*
	  printf("Elettroni Pi: %d\n",TotPi);
*/
	  for(k = 0;k < Ring;++k)  {

            if (AtmMtr[k] -> Ring > 0) AtmMtr[k] -> Ring *= -1;
/*
	    printf("%d %.4s %d %.3s_%-4.4s\n",
	           AtmMtr[k] -> Ring, AtmMtr[k] -> Name.C, AtmMtr[k] -> NSost,
		   AtmMtr[k] -> ResName.C, AtmMtr[k] -> ResSeq.C);
*/

	  }
	}
      }
    }
    Atm = Atm -> Ptr;
  }
}


/**** Descriptor conversion ****/

BOOL ToDesc(FFPAR *ParFF, FFDESC *Desc, char *Ptr)
{
  BOOL		Ret = TRUE;

  if (Ptr[1] != '-') Desc -> Atom.C[1] = Ptr[1];
  else Desc -> Atom.S  = (WORD)NULL;
  *Desc -> Atom.C = *Ptr;
  if ((Ptr[2] >= '0') && (Ptr[2] <= '9')) {
    Desc -> NSost = Ptr[2] - '0';
    if (((Ptr[3] >= '3') && (Ptr[3] <= '6')) ||
        (Ptr[3] == '9') || (Ptr[3] == '0')) {
      Desc -> Ring = Ptr[3] - '0';
      if ((Ptr[4] == '0') || (Ptr[4] == '1')) Desc -> Arom = Ptr[4] - '0';
      else {
        FFError(ParFF, "Illegal aromatic indicator");
        Ret = FALSE;
      }
    } else {
      FFError(ParFF, "Illegal ring indicator");
      Ret = FALSE;
    }
  } else {
    FFError(ParFF, "Illegal bond order");
    Ret = FALSE;
  }

  return Ret;
}
