/*

             **************************************************
             ****       VEGA - Format Definitions V1.1     ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************


*/

#ifndef __FORMATS
#define  __FORMATS

/**** File format types ****/

#define  FORM_TYPE_MOLE      65536  /* Molecule                           */
#define  FORM_TYPE_SURF     131072  /* Surface                            */
#define  FORM_TYPE_PLOT     262144  /* Plot                               */
#define  FORM_TYPE_CALC     524288  /* Calculation                        */

/**** File formats ****/

#define  FORM_PDB               (  0 | FORM_TYPE_MOLE)
#define  FORM_PDB_NONSTD        (  1 | FORM_TYPE_MOLE)
#define  FORM_BIOSYM_OLD        (  2 | FORM_TYPE_MOLE)
#define  FORM_BIOSYM            (  3 | FORM_TYPE_MOLE)
#define  FORM_IFF               (  4 | FORM_TYPE_MOLE)
#define  FORM_CSSR              (  5 | FORM_TYPE_MOLE)
#define  FORM_MOPINT            (  6 | FORM_TYPE_MOLE)
#define  FORM_XYZ               (  7 | FORM_TYPE_MOLE)
#define  FORM_PDB_Q             (  8 | FORM_TYPE_MOLE)
#define  FORM_QUANTA_SRF        (  9 | FORM_TYPE_SURF)
#define  FORM_BIOSYM_SRF        ( 10 | FORM_TYPE_SURF)
#define  FORM_PDB_FAT           ( 11 | FORM_TYPE_MOLE)
#define  FORM_QMC               ( 12 | FORM_TYPE_MOLE)
#define  FORM_QUANTA_MSF        ( 13 | FORM_TYPE_MOLE)
#define  FORM_VRML_SRF_POINTS   ( 14 | FORM_TYPE_MOLE)
#define  FORM_VRML_SRF_CPK      ( 15 | FORM_TYPE_MOLE)
#define  FORM_VRML              ( 16 | FORM_TYPE_MOLE)
#define  FORM_VRML_SOLID        ( 17 | FORM_TYPE_MOLE)
#define  FORM_INFO              ( 18 | FORM_TYPE_CALC)
#define  FORM_FASTA             ( 19 | FORM_TYPE_MOLE)
#define  FORM_ENER_CVFF         ( 20 | FORM_TYPE_CALC)
#define  FORM_MOL2              ( 21 | FORM_TYPE_MOLE)
#define  FORM_CRD               ( 22 | FORM_TYPE_MOLE)
#define  FORM_GROMOS            ( 23 | FORM_TYPE_MOLE)
#define  FORM_QUANTA_LOGP       ( 24 | FORM_TYPE_SURF)
#define  FORM_QUANTA_MEP        ( 25 | FORM_TYPE_SURF)
#define  FORM_GROMOS_NM         ( 26 | FORM_TYPE_MOLE)
#define  FORM_PLOT_CSV          ( 27 | FORM_TYPE_PLOT)
#define  FORM_PLOT_QUANTA       ( 28 | FORM_TYPE_PLOT)
#define  FORM_PLOT_BIN          ( 29 | FORM_TYPE_PLOT)

/**** Surface formats ****/

#define  SRF_QUANTA              0
#define  SRF_INSIGHT             1
#define  SRF_VRML_POINTS         2
#define  SRF_VRML_SOLID          3
#define  SRF_QUANTA_LOGP         4
#define  SRF_QUANTA_MEP          5

/**** PDB subformats ****/

#define  PDB_PURE                0
#define  PDB_NONSTD              1
#define  PDB_PDBQ                2
#define  PDB_FAT                 3

/**** Biosym subformats ****/

#define  BIOSYM_OLD              1
#define  BIOSYM_NEW              3

/**** CSSR subformats ****/

#define  CSSR_PURE               0
#define  CSSR_QMC                1

/**** Trajectory mesure ****/

#define  TRJMS_DISTANCE          0      /* Please don't chenge the order */
#define  TRJMS_ANGLE             1
#define  TRJMS_TORSION           2
#define  TRJMS_PLANEANG          3
#define  TRJMS_VLOGP             4
#define  TRJMS_DIPOLE            5
#define  TRJMS_SURFACE           6
#define  TRJMS_SURFDIA           7
#define  TRJMS_VOLUME            8
#define  TRJMS_VOLDIA            9


/**** Control flags for file formats ****/

#define  FORM_FLAG_CONN         1  /* Allow the connectivity calculation */
#define  FORM_FLAG_CONNAL       2  /* Calculate always the connectivity  */
#define  FORM_FLAG_FFCALC       4  /* Allow the force field attribuition */
#define  FORM_FLAG_CHARGE       8  /* Allow atomic charges attribuition  */
#define  FORM_FLAG_STDOUT      16  /* Allow the stdout use               */
#define  FORM_FLAG_CHKCHG      32  /* Check the charges                  */
#define  FORM_FLAG_CHKFF       64  /* Check the force field              */

/**** Shape types for solvatation ****/

#define  SOL_SHAPE_BOX          0
#define  SOL_SHAPE_SPHERE       1
#define  SOL_SHAPE_LAYER        2


#ifdef __MAIN
FILEFORMAT FileInfo[] = {
/* PDB             */ { FORM_PDB            , "PDB"       , "pdb" , FORM_FLAG_STDOUT|FORM_FLAG_CONN },
/* PDB Not Std.    */ { FORM_PDB_NONSTD     , "PDBNOTSTD" , "pdb" , FORM_FLAG_STDOUT|FORM_FLAG_CONN },
/* Old Biosym      */ { FORM_BIOSYM_OLD     , "OLDBIOSYM" , "car" , FORM_FLAG_STDOUT|FORM_FLAG_FFCALC|FORM_FLAG_CHARGE|FORM_FLAG_CHKCHG|FORM_FLAG_CHKFF },
/* New Biosym      */ { FORM_BIOSYM         , "BIOSYM"    , "car" , FORM_FLAG_STDOUT|FORM_FLAG_FFCALC|FORM_FLAG_CHARGE|FORM_FLAG_CHKCHG|FORM_FLAG_CHKFF },
/* IFF             */ { FORM_IFF            , "IFF"       , "iff" , FORM_FLAG_CONN|FORM_FLAG_FFCALC|FORM_FLAG_CHARGE|FORM_FLAG_CHKCHG|FORM_FLAG_CHKFF },
/* CSSR            */ { FORM_CSSR           , "CSSR"      , "cssr", FORM_FLAG_STDOUT|FORM_FLAG_CONN|FORM_FLAG_CONNAL|FORM_FLAG_CHARGE|FORM_FLAG_CHKCHG },
/* Mopac Int.      */ { FORM_MOPINT         , "MOPINT"    , "dat" , FORM_FLAG_STDOUT|FORM_FLAG_CONN|FORM_FLAG_CHARGE },
/* Xyz             */ { FORM_XYZ            , "XYZ"       , "xyz" , FORM_FLAG_STDOUT },
/* PDBQ            */ { FORM_PDB_Q          , "PDBQ"      , "pdb" , FORM_FLAG_STDOUT|FORM_FLAG_CONN|FORM_FLAG_CHARGE|FORM_FLAG_CHKCHG },
/* Quanta Surface  */ { FORM_QUANTA_SRF     , "QUANTASRF" , "srf" , 0 },
/* Insight Surf.   */ { FORM_BIOSYM_SRF     , "BIOSYMSRF" , "srf" , FORM_FLAG_STDOUT },
/* PDB Fat         */ { FORM_PDB_FAT        , "PDBF"      , "pdb" , FORM_FLAG_STDOUT|FORM_FLAG_CONN|FORM_FLAG_FFCALC|FORM_FLAG_CHARGE|FORM_FLAG_CHKCHG|FORM_FLAG_CHKFF },
/* QMC             */ { FORM_QMC            , "QMC"       , "qmc" , FORM_FLAG_STDOUT|FORM_FLAG_CONN|FORM_FLAG_CONNAL|FORM_FLAG_CHARGE|FORM_FLAG_CHKCHG },
/* Quanta Msf      */ { FORM_QUANTA_MSF     , "MSF"       , "msf" , FORM_FLAG_CONN|FORM_FLAG_FFCALC|FORM_FLAG_CHARGE|FORM_FLAG_CHKCHG },
/* VRML Dot Surf.  */ { FORM_VRML_SRF_POINTS, "VRMLPTS"   , "wrl" , FORM_FLAG_STDOUT },
/* VRML CPK        */ { FORM_VRML_SRF_CPK   , "VRMLCPK"   , "wrl" , FORM_FLAG_STDOUT },
/* VRML wireframe  */ { FORM_VRML           , "VRML"      , "wrl" , FORM_FLAG_STDOUT|FORM_FLAG_CONN|FORM_FLAG_CONNAL },
/* VRML Solid Srf  */ { FORM_VRML_SOLID     , "VRMLSOL"   , "wrl" , FORM_FLAG_STDOUT },
/* Informations    */ { FORM_INFO           , "INFO"      , "inf" , FORM_FLAG_STDOUT|FORM_FLAG_CONN|FORM_FLAG_CONNAL|FORM_FLAG_CHARGE|FORM_FLAG_CHKCHG },
/* Fasta           */ { FORM_FASTA          , "FASTA"     , "fas" , FORM_FLAG_STDOUT },
/* Inter. Enargy   */ { FORM_ENER_CVFF      , "CVFF"      , "ene" , FORM_FLAG_STDOUT|FORM_FLAG_FFCALC|FORM_FLAG_CHARGE|FORM_FLAG_CHKCHG|FORM_FLAG_CHKFF },
/* Sybyl Mol 2     */ { FORM_MOL2           , "MOL2"      , "mol2", FORM_FLAG_STDOUT|FORM_FLAG_CONN|FORM_FLAG_CONNAL|FORM_FLAG_CHARGE|FORM_FLAG_CHKCHG|FORM_FLAG_CHKFF },
/* CHARMm CRD      */ { FORM_CRD            , "CRD"       , "CRD" , FORM_FLAG_STDOUT|FORM_FLAG_CHARGE|FORM_FLAG_CHKCHG },
/* Gromos          */ { FORM_GROMOS         , "GROMOS"    , "gro" , FORM_FLAG_STDOUT },
/* Quanta LogP Srf */ { FORM_QUANTA_LOGP    , "QUANTALOGP", "srf" , FORM_FLAG_CONN|FORM_FLAG_CONNAL},
/* Quanta Mep Srf  */ { FORM_QUANTA_MEP     , "QUANTAMEP" , "srf" , FORM_FLAG_CHARGE|FORM_FLAG_CHKCHG },
/* Gromos nm       */ { FORM_GROMOS_NM      , "GROMOSNM"  , "gro" , FORM_FLAG_STDOUT },
/* CSV Plot        */ { FORM_PLOT_CSV       , "CSV"       , "csv" , FORM_FLAG_STDOUT|FORM_FLAG_CONN|FORM_FLAG_CHARGE },
/* Quanta Plot     */ { FORM_PLOT_QUANTA    , "QUANTAPLT" , "plt" , FORM_FLAG_STDOUT|FORM_FLAG_CONN|FORM_FLAG_CHARGE },
/* Binary Plot     */ { FORM_PLOT_BIN       , "BINPLT"    , "bin" , FORM_FLAG_STDOUT|FORM_FLAG_CONN|FORM_FLAG_CHARGE },
                      { 0                   , NULL        , NULL  , 0}
};
#else
extern FILEFORMAT  FileInfo[];
#endif
#endif


