/*

             **************************************************
             ****             VEGA - Info V1.3             ****
             ****                Saver only                ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************

*/

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "globdef.h"
#include "globvar.h"


/**** Info viewer ****/

BOOL InfoSave(FILE *FH, ATOMO *InizAtm, ULONG TotAtm)
{
  BOOL              CalcChrg;
  float             AAMass[20], LogP, MaxRad, MomLip;
  float             *Contrib, *VdwRad;
  register ATMTAB   *Table;
  register ATOMO    *Atm;
  register UWORD    AA;
  ULONG             AAComp[20], Nu;
  XYZ               BoxMin, BoxMax;
  XYZ               *SfUn;

  BOOL              Ret         = TRUE;
  float             Charge      = 0.0;
  float             Gx          = 0.0;
  float             Gy          = 0.0;
  float             Gz          = 0.0;
  float             Mass        = 0.0;
  float             Mx          = 0.0;
  float             My          = 0.0;
  float             Mz          = 0.0;
  LONG              PrecResName = 0;
  LONG              PrecResSeq  = 0;
  ULONG             HeavyAtm    = 0;
  ULONG             Mols        = 0;
  ULONG             Res         = 0;
  ULONG             TotAA       = 0;
  ULONG             Waters      = 0;


  if (Contrib = (float *)Alloca(TotAtm * sizeof(float))) {
    if (VdwRad = AssignVdwRad(InizAtm, TotAtm, 0.0, &MaxRad)) {
        for(AA = 0; AA < 20; ++AA) {
          AAComp[AA] = 0;
          AAMass[AA] = 0;
        }
        RenameWat(InizAtm);
        CalcChrg = ChkCharges(InizAtm, TotAtm, &Charge);

        if (fprintf(FH, "\n\n" \
                        "******************************\n" \
                        "**** Infos about molecule ****\n" \
                        "******************************\n\n" \
                        "Atoms..............: %d\n", TotAtm) > 0) {

          for(Atm = InizAtm; Atm; Atm = Atm -> Ptr) {
            if ((Atm -> ResName.L != PrecResName) || (Atm -> ResSeq.L != PrecResSeq)) {
              for(AA = 0; (AA < 20) && (Atm -> ResName.L != AAResTab[AA].L); ++AA);
              PrecResName = Atm -> ResName.L;
              PrecResSeq  = Atm -> ResSeq.L;
              ++Res;
              if (AA < 20) {
                ++AAComp[AA];
                ++TotAA;
              }
            }
            if (Atm -> Flags >= 2) ++Mols;
            if (Atm -> Elem.S != (*(WORD *)"H")) ++HeavyAtm;
            if ((Atm -> ResName.L == *((LONG *)"HOH")) &&
                (Atm -> Elem.S == (*(WORD *)"O"))) ++Waters;
            for(Table = AtmTable;*Table -> Atom.C; ++Table)
              if (Atm -> Elem.S == Table -> Atom.S) {
                if (AA < 20) AAMass[AA] += Table -> Mass;
                Mass += Table -> Mass;
                Mx   += (Table -> Mass * Atm -> x);
                My   += (Table -> Mass * Atm -> y);
                Mz   += (Table -> Mass * Atm -> z);
                break;
              }
            Gx     += Atm -> x;
            Gy     += Atm -> y;
            Gz     += Atm -> z;
          } /* End of atom loop */

          /**** Dimension of the molecule ****/

          CalcBox(InizAtm, VdwRad, &BoxMin, &BoxMax);
          if (fprintf(FH, "Heavy atoms........: %d\n" \
                          "Residues...........: %d\n" \
                          "Molecules..........: %d\n" \
                          "Waters.............: %d\n" \
                          "Molecular weight...: %.3f Daltons\n" \
                          "Geometry center....: %.4f %.4f %.4f\n" \
                          "Mass center........: %.4f %.4f %.4f\n" \
                          "Appx. dimensions...: %.4f %.4f %.4f\n" \
                          , HeavyAtm, Res, Mols, Waters, Mass,
                          Gx / TotAtm, Gy / TotAtm, Gz / TotAtm,
                          Mx / Mass, My / Mass, Mz / Mass,
                          BoxMax.x - BoxMin.x, BoxMax.y - BoxMin.y, BoxMax.z - BoxMin.z) > 0) {
            if (CalcChrg) {
              if (fprintf(FH, "Total charge.......: %.4f\n" \
                              "Dipole.............: %.4f Debye\n",
                              Charge, CalcDipole(InizAtm)) <= 0)
                Ret = PrintDosErr();
            }

            /**** Surface area ****/

            if (Ret) {
              AddToRad(InizAtm, VdwRad, Prefs.SAS_PROBERAD);
              if (SfUn = CalcSfUn(Prefs.SAS_POINTS, MaxRad + Prefs.SAS_PROBERAD, &Nu)) {
                Gx = CalcSrfArea(InizAtm, VdwRad, SfUn, Nu, NULL, MaxRad + Prefs.SAS_PROBERAD);
                Gy = SQR(Gx / PI);
                if (fprintf(FH, "Surf. area (%3.2f)..: %.1f " C_ANGSTROM C_QUAD \
                                " (ds=%.1f " C_ANGSTROM ")\n",
                                Prefs.SAS_PROBERAD, Gx, Gy) > 0) {

                  /**** Molecular volume ****/

                  AddToRad(InizAtm, VdwRad, 0 - Prefs.SAS_PROBERAD);
                  Gx = CalcVolume(InizAtm, VdwRad, Prefs.VOL_DEN, &BoxMin, &BoxMax);
                  Gz = pow(6.0 * Gx / PI, 1.0 / 3.0);
                  if (fprintf(FH, "Volume.............: %.1f " C_ANGSTROM C_CUBE \
                                  " (dv=%.1f " C_ANGSTROM ")\n", Gx, Gz) >  0) {
                    if (fprintf(FH, "Ovality............: ") > 0) {
                      if (!Prefs.SAS_PROBERAD) {
                        if (fprintf(FH, "%.1f\n", Quad(Gy / 2.0) / Quad(Gz / 2)) <= 0)
                          Ret = PrintDosErr();
                      } else if (fprintf(FH, "Not available (use -g 0)\n") <= 0)
                        Ret =PrintDosErr();
                    } else Ret = PrintDosErr();
                  } else Ret = PrintDosErr();
                } else Ret = PrintDosErr();
                FREE(SfUn);
              } else Ret = FALSE;
            }

            /**** logP Ghose/Crippen ****/

            if ((Ret) && (LogPCalc(InizAtm, TotAtm, &LogP, &MomLip, "CRIPPEN", NULL))) {
              if (fprintf(FH, "logP (Crippen).....: %.4f\n" \
                              "Lipole (Crippen)...: %.4f\n",
                              LogP, MomLip) <= 0)
                Ret = PrintDosErr();
            }

            /**** logP Broto/Moreau ****/

            if ((Ret) && (LogPCalc(InizAtm, TotAtm, &LogP, &MomLip, "BROTO", Contrib))) {
              if (fprintf(FH, "logP (Broto).......: %.4f\n" \
                              "Lipole (Broto).....: %.4f\n",
                              LogP, MomLip) > 0) {

                 /**** Virtual logP ****/

#ifdef VLOGP
                 AddToRad(InizAtm, VdwRad, VLOGP_PROBE_RAD);
                 if (SfUn = CalcSfUn(VLOGP_DOT_DENSITY, MaxRad + VLOGP_PROBE_RAD, &Nu)) {
                   if (fprintf(FH, "Virtual logP.......: %.4f\n",
                               CalcVLogP(InizAtm, VdwRad, MaxRad + VLOGP_PROBE_RAD, SfUn,
                               Nu, Contrib)) <= 0)
                     Ret = PrintDosErr();
                   FREE(SfUn);
                 } else Ret = FALSE;
#else
                 if (fprintf(FH, "Virtual logP.......: Not available\n") <= 0)
                     Ret = PrintDosErr();

#endif
              } else Ret = PrintDosErr();
            }

            if ((Ret) && (TotAA)) {
              if (fprintf(FH, "Polypeptidic charge: %d\n"
                              "\n\nAminoacidic composition:\n\n" \
                              " Res    N.   N. %%     Mass   Mass %%\n" \
                              "====================================\n",
                              FindRes(InizAtm, TotAtm)) > 0) {
                for(AA = 0; (Ret) && (AA < 20); ++AA)
                  if (AAComp[AA]) {
                    if (fprintf(FH, " %-4s %4d  %5.2f %9.3f  %5.2f\n",
                                AAResTab[AA].C, AAComp[AA],
                                (float)((AAComp[AA]  * 100) / (float)TotAA), AAMass[AA],
                                AAMass[AA] * 100.0 / Mass) <= 0)
                      Ret = PrintDosErr();
                  }
              } else Ret = PrintDosErr();
            }
          } else Ret = PrintDosErr();
        } else Ret = PrintDosErr();



      FREE(VdwRad);
    } else Ret = FALSE;
    FREE(Contrib);
  } else Ret = FALSE;

  return Ret;
}
