/*

              **************************************************
              ****            VEGA - Loader V1.1            ****
              ****              Loader Module               ****
              **** Copyright 1996-2000, Alessandro Pedretti ****
              **************************************************



*/

#include <stdio.h>
#include <string.h>

#include "globdef.h"
#include "globvar.h"
#include "formats.h"
#include "iff.h"


/**** Autodetect subroutine ****/

ATOMO *Loader(char *MolFile, ULONG *TotAtomi, ULONG NMol)
{
  char    FileName[512];
  char    *Str;
  FILE    *MOLIN;
  RECORD  Line;
  ULONG   IFFLen, Num;

  ATOMO   *BegPtr = NULL;
  BOOL    Ret     = TRUE;
  ULONG   Count   = 0;

  *GlobSwitch     = 0;
  *TotAtomi       = 0;

  /**** Binary format recognition ****/

  if ((MOLIN = fopen(MolFile, "rb")) != NULL) {

    /**** Quanta/MSF ****/

    if (fread(Line.Line, sizeof(RECORD), 1, MOLIN) == 1) {;
      if (!strncmp(Line.Line + (sizeof(ULONG) * 4), "QUANTA", 6)) {
        PrintLoad("Quanta .msf", NMol);
        BegPtr = MsfLoad(MOLIN, TotAtomi);
        GLOBSW_IFFIIUB = TRUE;
        GLOBSW_IFFCALC = TRUE;

        /**** DCD Quanta/CHARMm trajectory file ****/

      } else if (!strncmp(Line.Line + sizeof(ULONG), "CORD", 4)) {
        Prefs.DCD_FILE = MolFile;
        strcpy(FileName, MolFile);
        ChangeExt(FileName, "CRD");
        fclose(MOLIN);

        /**** Read the associated CRD file ****/

        if (GLOBSW_MESTRJ) {
          if ((MOLIN = fopen(FileName, "r")) != NULL) {
            if ((fgets(Line.Line, LINELEN, MOLIN)) && (fgets(Line.Line, LINELEN, MOLIN))) {
              if (*Line.Line == '*') {
                PrintLoad("CHARMm CRD", NMol);
                BegPtr = CRDLoad(MOLIN, &Line, TotAtomi);
                GLOBSW_IFFCALC = TRUE;
              } else Ret = Errore("Unrecognized CRD file");
            } else Ret = PrintDosErr();
          } else Ret = Errore("Can't open the associated molecular file");
        } else Ret = Errore("Mesure mode not specified (-m option)");

      /**** IFF format ****/

      } else if (!strncmp(Line.Line, "FORM", 4)) {
        fseek(MOLIN, 4, SEEK_SET);
        if ((fread(&IFFLen  , sizeof(ULONG), 1, MOLIN) == 1) &&
            (fread(&Line.Hdr, sizeof(LONG) , 1, MOLIN) == 1)) {
#ifdef BIG_ENDIAN
          Swap(&IFFLen);
#endif
          if (!strncmp(Line.Line, "MOLE", 4)) {
            PrintLoad("IFF molecular", NMol);
            BegPtr = IFFLoad(MOLIN, IFFLen, TotAtomi);
            GLOBSW_IFFIIUB = TRUE;
          }
        } else Ret = PrintDosErr();
      } else {
        fclose(MOLIN);

        /**** ASCII format recognition ****/

        if ((MOLIN = fopen(MolFile, "r")) != NULL) {
          while(fgets(Line.Line, LINELEN, MOLIN)) {

              /**** Biosym Archive 1 & 3 format ****/

            if (!strncmp("!BIOSYM archive", Line.Line, 15)) {
              PrintLoad("Biosym .car", NMol);
              BegPtr = CarLoad(MOLIN, &Line, TotAtomi);
              GLOBSW_IFFIIUB = TRUE;
              GLOBSW_IFFCALC = TRUE;
              break;
            }

            /**** PDB format ****/

            if ((Line.Hdr == *(LONG *)PDBAtmRec[0]) ||
                (Line.Hdr == *(LONG *)PDBAtmRec[1]) ||
                (Line.Hdr == *(LONG *)"REMA")) {
              PrintLoad("PDB", NMol);
              BegPtr = PDBLoad(MOLIN, &Line, TotAtomi);
              GLOBSW_IFFIIUB = TRUE;
              break;
            }

            /**** CSSR format ****/

            if ((Count == 4) && (!strncmp(Line.Line,"   1 ",5)) &&
                (!strncmp(Line.Line + 9, "  ",2))) {
              PrintLoad("CSSR", NMol);
              BegPtr        = CSSRLoad(MOLIN, &Line, TotAtomi, CSSR_PURE);
              GLOBSW_IFFCALC = TRUE;
              break;
            }

            /**** QMC format ****/

            if ((Count == 2) && (!strncmp(Line.Line,"   1 ",5)) &&
               (Line.Line[15] == '.') && (Line.Line[26] == '.') &&
               (Line.Line[37] == '.') && (Line.Line[73] == '.')) {
              PrintLoad("QMC", NMol);
              BegPtr        = CSSRLoad(MOLIN, &Line, TotAtomi, CSSR_QMC);
              GLOBSW_IFFCALC = TRUE;
              break;
            }

            /**** MoPac .arc format ****/

            if (!strncmp(Line.Line, "          FINAL GEOMETRY OBTAINED", 33)) {
              PrintLoad("MoPac Arc", NMol);
              BegPtr = MoPacArcLoad(MOLIN, &Line, TotAtomi);
              GLOBSW_IFFCALC = TRUE;
              break;
            }

            /**** MoPac Internal format ****/

            if ((Count == 3) &&
                (!strncmp(Line.Line + 6, "0.0000000  0      0.000000  0      0.000000  0    0    0    0", 61))) {
              fseek(MOLIN, 0, SEEK_SET);
              PrintLoad("MoPac internal", NMol);
              BegPtr = MoPacArcLoad(MOLIN, &Line, TotAtomi);
              GLOBSW_IFFCALC = TRUE;
              break;
            }

            /**** IFF format ****/

            if ((!Count) && (Line.Hdr == IFF_FORM)) {
              fseek(MOLIN, 4, SEEK_SET);
              fread(&IFFLen  , sizeof(ULONG), 1, MOLIN);
              fread(&Line.Hdr, sizeof(LONG) , 1, MOLIN);
              if (Line.Hdr == IFF_MOLE) {
                PrintLoad("IFF molecular", NMol);
                BegPtr = IFFLoad(MOLIN, IFFLen, TotAtomi);
                GLOBSW_IFFIIUB = TRUE;
              }
              break;
            }

            /**** Mol2 format ****/

            if (!strcmp(Line.Line, "@<TRIPOS>MOLECULE\n")) {
              PrintLoad("Mol2", NMol);
              BegPtr = Mol2Load(MOLIN, &Line, TotAtomi);
              GLOBSW_IFFCALC = TRUE;
              break;
            }

            /**** CHARMm CRD format ****/

            if ((Count == 1) && (*Line.Line == '*')) {
              PrintLoad("CHARMm CRD", NMol);
              BegPtr = CRDLoad(MOLIN, &Line, TotAtomi);
              GLOBSW_IFFCALC = TRUE;
              break;
            }

            /**** Gromos format ****/

            if (!strcmp(Line.Line, "POSITION\n")) {
              PrintLoad("Gromos", NMol);
              BegPtr = GromosLoad(MOLIN, &Line, TotAtomi);
              GLOBSW_IFFCALC = TRUE;
              break;
            }

            /**** XYZ format ****/

            if (*Line.Line != '\n') {
              Num = 1;
              for(Str = Line.Line; *Str; ++Str) {
                if ((*Str >= 'A') && (*Str <= 'Z')) {
                  ++Str;
                  if (((*Str >= 'A') && (*Str <= 'Z')) || (*Str == ' ') ||
                      (*Str == '\t')) {
                    ++Str;
                    if (*Str == ' ') {
                      ++Str;
                      while(*Str) {
                        if ((*Str >= '0') && (*Str <= '9')) ++Num;
                        else if ((*Str != ' ') && (*Str != '\t') && (*Str != '.') &&
                                 (*Str != '+') && (*Str != '-') && (*Str != '\n')) {
                          Num = 0;
                          break;
                        }
                        ++Str;
                      }
                      if (!Num) break;
                    }
                  }
                }
              }
              if (Num >= 16) {
                PrintLoad("XYZ", NMol);
                BegPtr = XyzLoad(MOLIN, &Line, TotAtomi);
                break;
              }
            }
            ++Count;
          }
        }
      }

      fclose(MOLIN);
      if (BegPtr == (void *)-1) BegPtr = 0;
      else if ((!BegPtr) && (Ret)) Errore("Unknown input format");
    } else PrintDosErr();
  } else Errore("Input file not found");

  return BegPtr;
}



