/*

             ********************************************
             ****        VEGA V1.1 - Main code       ****
             **** (c) 1996-2000, Alessandro Pedretti ****
             ********************************************


*/

#include "main.h"

/**** Local variables ****/

static char		**LastArg;

/**** Local prototypes ****/

void     Sintassi(void);

/**** Main code ****/

int main(int argc, char *argv[])
{
  ATOMO          *Inizio, *LastSeg, *TmpIniz;
  char          Buf[256], FField[256], FCharge[256], Shell[256], Opt[100];
  char          **Arg, **CurFile, **Files;
  FILE          *TEST;
  FILEFORMAT    *OutFormat;
  TRJPAR        TrjPar;
  ULONG         k, TmpTotAtm, TotAtm;
  UWORD         CharFormat;

  BOOL          Ret          = TRUE;
  BYTE          Switch[]     = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
  UWORD         TotFiles     = 0;
  static char   *CharType[]  = { "FRAGMENTS", "GASTEIGER", "" };


  /**** Check the VEGADIR environment variable ****/

  if (!GetPrgPath(FField)) exit(0);
  strcpy(FCharge, FField);
  strcpy(Shell, FField);
  strcat(Shell, CLUSTER_DIR);
  strcat(FCharge, PREFS_FILE);
  LoadPrefs(FCharge);
  strcpy(FCharge, FField);
  OutFormat = Prefs.OUTFORMAT;

  if ((argc == 1) || (*argv[1] == '?')) Sintassi();

  if (Files = (char **)Alloca(argc * sizeof(char *))) {

    Arg           = &argv[0];
    CurFile       = Files;
    GlobSwitch[1] = TRUE;           /* Saves connectivity         */
    GlobSwitch[2] = (UBYTE)FALSE;   /* Don't saves IFF CALC chunk */
    LastArg       = &argv[argc];
    *OutFile      = 0;
    strcpy(FFName, "UNKNOWN");

    do {
      ++Arg;
      if (**Arg == '-') {
        if (strlen(*Arg) > 2) {
          Errore("Unknown option");
          Sintassi();
        }
        switch(tolower(*(*Arg + 1))) {

        case 'a':			/* Renumber residues          */
          Switch[SW_RENRES] = TRUE;
          if (SubArg(Arg + 1, SUBA_POSINT, FALSE)) {
            sscanf(*++Arg, "%d", &Prefs.RENSTART);
            if (Prefs.RENSTART < 1) {
              Errore("Illegal starting residue");
              Ret = FALSE;
            }
          }
          break;

        case 'b':			/* Don't save connectivity    */
          GlobSwitch[1]            = FALSE;
          Switch[SW_NOCONNECT]     = TRUE;
          break;

        case 'c':			/* Assign template of charges */
          if (Ret = SubArg(++Arg, SUBA_TOUP, TRUE)) {
            strcat(FCharge, *Arg);
            strcat(FCharge, ".tem");
            Ret = FALSE;
            if (TEST = fopen(FCharge, "r")) {
              fgets(Buf, 255, TEST);
              if (!strncasecmp(Buf, "#TemplateCharge", 15)) {
                sscanf(Buf, "%*s %s", Opt);
                for(CharFormat = 0; *CharType[CharFormat]; ++CharFormat)
                  if (!strcasecmp(CharType[CharFormat], Opt)) {
                    Ret                = TRUE;
                    Switch[SW_CHARGES] = TRUE;
                    if (CharFormat != CHARGE_FRAG) Switch[SW_FINDRINGS] = TRUE;
                    break;
                  }
              }
              if (!Ret) Errore("Unsupported charge template");
              fclose(TEST);
            } else Errore("Can't find charge template");
          }
          break;

        case 'd':			/* Dielectric constant        */
          if (Ret = SubArg(++Arg, SUBA_POSFLOAT, TRUE)) {
            sscanf(*Arg, "%f", &Prefs.ENDIEL);
            if (Prefs.ENDIEL <= 0) {
              Errore("Illegal dielectric constant");
              Ret = FALSE;
            }
          }
          break;

        case 'e':			/* Residue name for energy    */
          if (Ret = SubArg(++Arg, SUBA_NOCHK, TRUE))
            if (!GetPrefsRes(*Arg)) Errore("Illegal residue name");
          break;

        case 'f':			/* Output format              */
          if (Ret = SubArg(++Arg, SUBA_NOCHK, TRUE)) {
            for(OutFormat = FileInfo;(OutFormat -> Com) &&
               (strcasecmp(*Arg, OutFormat -> Com));++OutFormat);
            if (!OutFormat -> Com) Ret = Errore("Unknown output format");
          }
          break;

        case 'g':			/* Probe radius for SAS       */
          if (Ret = SubArg(++Arg, SUBA_POSFLOAT, TRUE)) {
            sscanf(*Arg, "%f", &Prefs.SAS_PROBERAD);
            if (Prefs.SAS_PROBERAD < 0) {
              Errore("Illegal probe radius");
              Ret = FALSE;
            }
          }
          break;

        case 'i':			/* Solvate the molecule	      */
          Switch[SW_SOLVATE] = TRUE;
          if (Ret = SubArg(++Arg, SUBA_TOUP, TRUE)) {
            strcat(Shell, *Arg);
            strcat(Shell, ".pdb");
            if (TEST = fopen(Shell, "r")) {
              fclose(TEST);
              if (SubArg(++Arg, SUBA_POSFLOAT, FALSE)) {
                sscanf(*Arg, "%f", &Prefs.SOL_RAD);
                if (Prefs.SOL_RAD == 0) {
                  Errore("Illegal solvatation radius");
                  Ret = FALSE;
                } else if (SubArg(++Arg, SUBA_TOUP, FALSE)) {
                  Ret = FALSE;
                  for(k = 0; ShapeType[k]; ++k)
                    if (!strcmp(ShapeType[k], *Arg)) {
                      Prefs.SOL_SHAPE = k;
                      Ret = TRUE;
                    }
                  if (!Ret) Errore("Unknown shape type");
                } else --Arg;
              } else --Arg;
            } else {
              Errore("Shell file not found");
              Ret = FALSE;
            }
          }
          break;

        case 'k':			/* Keywords for Mopac         */
          if (Ret = SubArg(++Arg, SUBA_NOCHK, TRUE)) {
            strncpy(Prefs.MOPAC_DEF, *Arg, 80);
            Prefs.MOPAC_FLAG = TRUE;
          }
          break;

        case 'm':                       /* Analysis options           */
         if (Ret = SubArg(++Arg, SUBA_NOCHK, TRUE)) {
           TrjPar.Arg = Arg;
           TrjPar.Num = 1;
	   if (*Arg != (char *)argv[argc - 1]) {
             do {
	       ++Arg;
               if (**Arg == '-') {
                 --Arg;
                 break;
               } else ++TrjPar.Num;
             } while(*Arg != (char *)argv[argc - 1]);
	   }
           Ret = TrjParser(&TrjPar);
           GLOBSW_MESTRJ = TRUE;
         }
         break;

        case 'n':			/* Normalize coordinates      */
          Switch[SW_NORMALIZE] = TRUE;
          break;

        case 'o':			/* Output file name           */
          if (Ret = SubArg(++Arg, SUBA_NOCHK, TRUE))
            strcpy(OutFile, *Arg);
          break;

        case 'p':			/* Force field name           */
          if (Ret = SubArg(++Arg, SUBA_NOCHK, TRUE)) {
            strcpy(FFName, *Arg);
            Upper(FFName);
            strcat(FField, FFName);
            strcat(FField, ".tem");
            Ret = FALSE;
            if (TEST = fopen(FField, "r")) {
              fgets(Buf, 12, TEST);
              if (!strncasecmp(Buf, "#TemplateFF", 11)) {
                Ret                  = TRUE;
                Switch[SW_FFIELD]    = TRUE;
                Switch[SW_FINDRINGS] = TRUE;
                GlobSwitch[2]        = TRUE;
              } else Errore("Force field template is not in standard format");
              fclose(TEST);
            } else Errore("Unable to find the force field template");
          }
          break;

        case 'r':			/* Remove hydrogens           */
          Switch[SW_RMHYDROG] = TRUE;
          break;

        case 's':			/* Dot density for SAS        */
          if (Ret = SubArg(++Arg, SUBA_POSINT, TRUE)) {
            sscanf(*Arg, "%d", &Prefs.SAS_POINTS);
            if (Prefs.SAS_POINTS == 0) {
              Errore("Illegal SRS density");
              Ret = FALSE;
            }
          }
          break;

        case 'w':			/* Remove waters              */
          Switch[SW_RMWATER] = TRUE;
          break;

        case 'h':			/* Help                       */
        default:
          Sintassi();
        }
      } else {
        ++TotFiles;
        *CurFile = *Arg;
        ++CurFile;
      }

    } while ((Ret) && (*Arg != argv[argc - 1]));

    if (Ret) {
      if (TotFiles) {
        if (!*OutFile) {
          GlobSwitch[4] = TRUE;
          if (!(OutFormat -> Flags & FORM_FLAG_STDOUT)) {
            Errore("stdout not allowed with selected format");
            Ret = FALSE;
          }
        }
        if (Ret) {
          if ((Switch[SW_FFIELD]) && (!(OutFormat -> Flags & FORM_FLAG_FFCALC)))
            Errore("The output format don't support force field tag");
          else {
            if ((Switch[SW_CHARGES]) && (!(OutFormat -> Flags & FORM_FLAG_CHARGE)))
              Errore("The output format don't support atomic charges");
            else {

              if ((!Switch[SW_FFIELD]) && (!Switch[SW_RMHYDROG]) &&
                  (!Switch[SW_RMWATER]) && (!Switch[SW_CHARGES])) {
                if ((!Switch[SW_NOCONNECT]) || (OutFormat -> Flags & FORM_FLAG_CONNAL) ||
                    (TrjPar.Com == TRJMS_VLOGP)){
                  if (OutFormat -> Flags & FORM_FLAG_CONN)
                    GlobSwitch[3] = TRUE;
                  else GlobSwitch[3] = FALSE;
                }
              } else GlobSwitch[3] = TRUE;

              /**** Multiple loads ****/

              TotAtm  = 0;
              CurFile = Files;
              for(k = 1; k <= TotFiles; ++k) {
                LastSeg   = LastAtm;
                TmpIniz   = Loader(*CurFile, &TmpTotAtm, k);
                if ((TmpIniz) && (TmpIniz != (ATOMO *)-1)) {
                  TotAtm += TmpTotAtm;
                  if (k != 1) LastSeg -> Ptr = TmpIniz;	/* Binding */
                 else Inizio = TmpIniz;
                  if ((GlobSwitch[3]) && (!Connect(TmpIniz, TmpTotAtm))) {
                    Switch[SW_SAVMOL] = TRUE;
                    Ret = FALSE;
                    break;
                  }
                } else {
                  Ret = FALSE;
                  break;
                }
                ++CurFile;
              }

              if (Ret) {
                if (((OutFormat -> Id == FORM_CSSR) || (OutFormat  -> Id == FORM_QMC )) && (TotAtm > 9999)) {
                  Ret = Errore("CSSR/QMC format don't support over 9999 atoms");
                  Switch[SW_SAVMOL] = TRUE;
                } else {
                  if (Switch[SW_NORMALIZE]) Normalize(Inizio, TotAtm);
                  if (Switch[SW_RENRES]) RenRes(Inizio);
                  if (Switch[SW_RMWATER]) {
                    RemoveH2O(&Inizio, &TotAtm);
                    if (!TotAtm) {
                      Switch[SW_SAVMOL] = TRUE;
                      Ret = FALSE;
                    }
                  }
                  if (Switch[SW_SOLVATE])
                    Ret = Solvate(Shell, Inizio, &TotAtm);

                  if (Ret) {
                    if ((Switch[SW_FINDRINGS]) || (OutFormat -> Id == FORM_MOL2) ||
                        (OutFormat -> Id == FORM_INFO) || (OutFormat -> Id == FORM_QUANTA_LOGP))
                      FindRing(Inizio);
                    if ((Switch[SW_CHARGES]) && (!Charge(FCharge, CharFormat, Inizio, TotAtm)))
                      Switch[SW_SAVMOL] = TRUE;
                    else if ((Switch[SW_FFIELD]) && (!AssignFF(FField, Inizio, TotAtm, FALSE)))
                      Switch[SW_SAVMOL] = TRUE;
                  }
                }
                if (!Switch[SW_SAVMOL]) {
                  if (Switch[SW_RMHYDROG]) RemoveH(&Inizio, &TotAtm);
                  if (Prefs.DCD_FILE) {
                    PrintProg("* Analyzing DCD trajectory");
                    TrjPar.OutFile = OutFile;
                    TrjPar.OutForm = OutFormat;
                    Ret = DCDAnalyzer(Prefs.DCD_FILE, Inizio, TotAtm, &TrjPar);
                  } else Ret = Saver(OutFile, OutFormat, Inizio, TotAtm);
                  if (Ret) PrintProg("Done.\n");
                } else PrintProg("Not saved.\n");
                PDBFree(Inizio);
              }
            }
          }
        }
      } else Errore("Missing input file name");
    }
    FREE(Files);
  }

  return 0;
}


/**** Show the correct syntax ****/

void Sintassi(void)
{
#ifdef WIN32
  printf("\nVEGA V" VEGA_VER " - (c) " VEGA_COPYRIGHT \
                  ", Alessandro Pedretti & Giulio Vistoli\n");
#else
  printf("\n\033[33mVEGA V" VEGA_VER "\033[0m - (c) " VEGA_COPYRIGHT \
                  ", Alessandro Pedretti & Giulio Vistoli\n");
#endif

#ifdef VLOGP
  printf("Virtual logP by Bernard Testa et al.\n");
#endif

#ifdef AMIGA
#ifdef mc68040
  printf("AmigaDOS 68040 version.\n\n");
#else

#ifdef mc68030
#ifdef __HAVE_68881__
  printf("AmigaDOS 68030/68881 version.\n\n");
#else
  printf("AmigaDOS 68030 version.\n\n");
#endif
#else

#ifdef mc68020
#ifdef __HAVE_68881__
  printf("AmigaDOS 68020/68881 version.\n\n");
#else
  printf("AmigaDOS 68020 version.\n\n");
#endif
#else

#ifdef mc68000
  printf("AmigaDOS 68000 version.\n\n");
#endif
#endif
#endif
#endif

#else
#ifdef WIN32

#ifdef _M_IX86
#if _M_IX86 == 300
#define  CPUOPT         "386 "
#else
#if _M_IX86 == 400
#define  CPUOPT         "486 "
#else
#if _M_IX86 == 500
#define  CPUOPT         "Pentium "
#else
#if _M_IX86 == 600
#define  CPUOPT         "Pentium Pro "
#endif
#endif
#endif
#endif
#else
#define  CPUOPT
#endif
  printf("Windows 9x/NT " CPUOPT "version.\n\n");
#else
#ifdef  _SGI_SOURCE
  printf("Silicon Graphics version.\n\n");
#else
  printf("UNIX version.\n\n");
#endif
#endif
#endif

  printf("Synopsis: vega INPUT ... -o[OUTPUT] -f[OUTPUT_FORMAT] -p[FORCE_FIELD]\n" \
         "          -s[POINTS] -g[RADIUS] -c[TEMPLATE] -k[KEYWORDS] -a[RES_NUM]\n" \
         "          -d[DIELECTRIC] -e[NAME:NUM] -i[SHELL RAD SHAPE] -m[KEYWORDS]\n" \
         "          -bnrw\n\n" \
         " a -> renumber residues starting from RES_NUM\n" \
         " b -> don't save the connectivity\n" \
         " c -> charge template\n" \
         " d -> dielectric constant for energy calculation\n" \
         " e -> residue for energy calculation\n" \
         " f -> output format\n" \
         " g -> probe radius for SAS\n" \
         " h -> show this help\n" \
         " i -> solvate the molecule\n" \
         " k -> keywords for Mopac\n" \
         " n -> normalize coordinates\n" \
         " m -> keywords for trajectory analysis\n" \
         " o -> output file name\n" \
         " p -> define force field to apply\n" \
         " r -> remove hydrogens\n" \
         " s -> point density for SAS\n" \
         " w -> remove waters\n" \
         "\n" \
         "INPUT formats:\n" \
         " Biosym, CHARMm CRD, CHARMm DCD, CSSR, Gromos, IFF, Mol2, Mopac, MSF,\n" \
         " PDB, PDBF, QMC, XYZ.\n" \
         "\n" \
         "OUTPUT formats:\n" \
         " Calc:     CVFF, Info.\n" \
         " Molecule: CRD, CSSR, Fasta, Gromos, GromosNm, IFF, Mol2, MSF, PDB,\n" \
         "           PDBQ, PDBF, PDBNOTSTD, QMC, OldBiosym, Biosym, MopInt, \n" \
         "           XYZ.\n"
         " Plot:     BinPlt, CSV, QuantaPlt.\n" \
         " Surface:  BiosymSrf, QuantaLogP, QuantaMep, QuantaSrf.\n" \
         " VRML:     Vrml, VrmlPts, VrmlCpk, VrmlSol.\n" \
         "\n" \
         "TRAJECTORY keywords (-m):\n" \
         " Angle A1 A2 A3, Distance A1 A2, PlaneAng A1 A2 A3 A4 A5 A6, Dipole,\n" \
         " Surface A1 ..., SurfDia A1 ..., Torsion A1 A2 A3 A4, VlogP, Volume,\n" \
         " VolDia.\n" \
         "\n");
  exit(0);
}


/**** Extract arg options ****/

BOOL SubArg(char **PtrArg, WORD Opt, BOOL Mod)
{
  BOOL			Ret  = TRUE;
  register char		*Arg = *PtrArg;

  if ((PtrArg != LastArg) && (*Arg != '-')) {
    switch(Opt) {
    case SUBA_TOUP:         /* To upper                   */
      Upper(Arg);
      break;
    case SUBA_POSFLOAT:     /* Is positive float number   */
      do {
        if (((*Arg <  '0') || (*Arg > '9')) && (*Arg != '+') && (*Arg != '.')) {
          Ret = FALSE;
          if (Mod) Errore("The argument must be an unsigned float number");
        }
      } while((Ret) && (*++Arg));
      break;

    case SUBA_POSINT:      /* Is positive integer number */
      do {
        if (((*Arg <  '0') || (*Arg > '9')) && (*Arg != '+')) {
          Ret = FALSE;
          if (Mod) Errore("The argument must be an unsigned integer number");
        }
      } while((Ret) && (*++Arg));
      break;

    default:               /* Nothing                    */
      break;
    }
  } else {
    if (Mod) Errore("Missing sub-argument");
    Ret = FALSE;
  }

  return Ret;
}


