/*

             **************************************************
             ****             VEGA - MOL2 V1.0             ****
             ****              Loader & saver              ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************




*/

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "globdef.h"
#include "globvar.h"

#ifndef WIN32
#include <unistd.h>
#endif

#define  FIELDLEN           256
#define  TRIPOS_TEMPLATE    "TRIPOS.tem"


static char	*Hdr_Atom = "@<TRIPOS>ATOMO\n";


/**** Sybyl Mol2 Loader ****/

ATOMO *Mol2Load(FILE *IN, RECORD *Lin, ULONG *TotAtomi)
{
  ATOMO			*InizAtm;
  char			ResSeq[20];
  register ATOMO	*Atm;

  while(fgets(Lin -> Line, LINELEN, IN)) {
    if (!strcmp(Lin -> Line, Hdr_Atom)) {
      while((fgets(Lin -> Line, LINELEN, IN)) && (*Lin -> Line != '@')) {
        if ((*Lin -> Line) && (*Lin -> Line != '\n')) {
          if (Atm = AllocAtm(&InizAtm, TotAtomi)) {
            sscanf(Lin -> Line, "%*s %4s %f %f %f %4s %s %3s %f",
                   Atm -> Name.C, &Atm -> x, &Atm -> y, &Atm -> z,
                   Atm -> Pot.C, ResSeq, Atm -> ResName.C, &Atm -> Charge);
      	    ResConv(&Atm -> ResSeq, ResSeq);
            Atm -> Elem.C[0] = toupper(Atm -> Name.C[0]);
            if ((Atm -> Name.C[1]) && (islower(Atm -> Name.C[1])))
              Atm -> Elem.C[1] = Atm -> Name.C[1];
          } else break;
        }
      }
    }
  }

  if (Atm) {
    Atm -> Flags = 2;
    LastAtm      = Atm;
  } else Errore("Corrupted or incoplete Mol2 file");

  return InizAtm;
}


/**** Sybyl Mol2 saver ****/

BOOL Mol2Save(FILE *OUT, ATOMO *InizAtm, ULONG TotAtm, char *Name)
{
  char			Field[FIELDLEN], FieldName[FIELDLEN];
  char			*CharTyp, *LogName, *Ptr;
  FILE			*IN;
  QCHAR			Ord1, Ord2, Ord3, *Template, *TmpPtr;
  register ATOMO	*Atm;
  register UWORD	k;
  time_t		Orario;
  ULONG			Count;

  BOOL			Ret   = TRUE;

  /**** Load the force field template ****/

  GetPrgPath(FieldName);
  strcat(FieldName, TRIPOS_TEMPLATE);
  if (IN = (FILE *)fopen(FieldName, "r")) {
    Count = 0;
    while(fgets(Field, FIELDLEN, IN))
      if ((*Field != '\n') && (*Field != ';')) ++Count;
    fclose(IN);
    if (Count) {
      if (Template = (QCHAR *)Alloca((Count + 1) * sizeof(QCHAR))) {
        if (IN = (FILE *)fopen(FieldName, "r")) {
          TmpPtr = Template;
          while(fgets(Field, FIELDLEN, IN))
            if ((*Field != '\n') && (*Field != ';')) {
              sscanf(Field, "%4s", TmpPtr -> C);
              ++TmpPtr;
            }
          fclose(IN);

          /**** Check atom types ****/

          Count = 0;
          for(Atm = InizAtm; Atm; Atm = Atm -> Ptr) {
            for(TmpPtr = Template; TmpPtr -> L; ++TmpPtr)
              if (TmpPtr -> L == Atm -> Pot.L) {
                ++Count;
                break;
              }
            if (!TmpPtr -> L) break;
          }
          if (Count != TotAtm) {

            /**** Assign TRIPOS force field ****/

            PrintProg("  Assigning Tripos atom types");
            AssignFF(FieldName, InizAtm, TotAtm, TRUE);
          }
          if (*Name) {
            for(Ptr = Name + strlen(Name); Ptr != Name; --Ptr)
              if (*Ptr == '.') {
                *Ptr = 0;
                break;
              }
          } else Name = "Unknown";
#ifndef WIN32
          LogName = getlogin();
#else
          LogName = "User";
#endif
          Orario  = time(0);
          Ptr     = ctime(&Orario);
          if (ChkCharges(InizAtm, TotAtm, NULL)) CharTyp = "USER";
          else CharTyp = "NO";
          if (fprintf(OUT, "#       Name:                   %s\n" \
                           "#       Creating user name:     %s\n" \
                           "#       Creation time:          %s\n\n" \
                           "#       Modifying user name:    %s\n" \
                           "#       Modification time:      %s\n\n" \
                           "@<TRIPOS>MOLECULE\n%s\n" \
                           "%5d %5d    0    0    0\n" \
                           "SMALL\n%s_CHARGES\n\n%s\n" \
                           "@<TRIPOS>ATOMO\n",
                           Name, LogName, Ptr, LogName, Ptr, Name, TotAtm,
                           TotBond, CharTyp, VEGAHdr) > 0) {
            Count = 1;
            for(Atm = InizAtm; (Atm && Ret); Atm = Atm -> Ptr) {
              if (fprintf(OUT, "%7d %-8.4s %11.6f %11.6f %11.6f %-4.4s %6.4s %-8.4s %11.6f\n",
                               Count++, Atm -> Name.C, Atm -> x, Atm -> y, Atm -> z,
                               Atm -> Pot.C, Atm -> ResSeq.C, Atm -> ResName.C,
                               Atm -> Charge) <= 0) {
                PrintDosErr();
                Ret = FALSE;
              }
            }
            if (Ret) {
              if (fprintf(OUT, "@<TRIPOS>BOND\n") > 0) {
                Count = 1;
                for(Atm = InizAtm; (Atm && Ret); Atm = Atm -> Ptr) {
                  Ord1.L = AttToOrd(Atm);
                  for(k = 0; ((Ret) && (k < Atm -> NSost)); ++k) {
                    if (Atm -> Conn[k] -> Num > Atm -> Num) {
                      Ord2.L = AttToOrd(Atm -> Conn[k]);
                      if ((Ord1.L == 0x61720000) && (Ord2.L != 0x61720000)) Ord3.L = 0x31000000;
                      else if ((Ord2.L == 0x61720000) && (Ord1.L != 0x61720000)) Ord3.L = 0x31000000;
                           else if ((ULONG)(Ord1.L) >= (ULONG)(Ord2.L)) Ord3.L = Ord2.L;
                                else Ord3.L = Ord1.L;
                      if (fprintf(OUT, "%6d %5d %5d %.4s\n",
                                  Count++, Atm -> Num, Atm -> Conn[k] -> Num, Ord3.C) <= 0)
                        Ret = PrintDosErr();
                    }
                  }
                }
              } else Ret = PrintDosErr();
            }
          } else Ret = PrintDosErr();
        } else PrintDosErr();
        FREE(Template);
      }
    } else Ret = Errore("Illegal TRIPOS template");
  } else Ret = Errore("Can't find TRIPOS template");

  return Ret;
}


/**** Translate atom type to bond order ****/

ULONG AttToOrd(ATOMO *Atm)
{
  ULONG			Att;

  if (!Atm -> NSost) Att = 0x30000000;
  else Att = 0x31000000;

  if (Atm -> Pot.C[1] == '.') {
    if ((Atm -> Pot.C[2] == 'a') && (Atm -> Pot.C[3] == 'r')) Att = 0x61720000;
    else {
      if (Atm -> Pot.C[2] == '4') Att = 0x31000000;
      else Att = ((ULONG)(0x34 + 0x30 - Atm -> Pot.C[2])) << 24;
    }
  }

  return Att;
}
