/*

             **************************************************
             ****             VEGA - Fasta V1.0            ****
             ****                   Saver                  ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************



*/

#include <stdio.h>
#include <string.h>

#include "globdef.h"
#include "globvar.h"

#define  TYPETOL	0.8

/**** FASTA saver ****/

BOOL FastaSave(FILE *FH, ATOMO *InizAtm)
{
  char			*AATab1;
  register ATOMO        *Atm;
  register QCHAR	*AATab3;
  UWORD			Type;

  BOOL			Ret     = TRUE;
  LONG			PrecNum = 0;
  LONG			PrecRes = 0;
  ULONG			TotRes  = 0;

  if ((Type = TypeOfMol(InizAtm)) != MOL_UNK) {
    fprintf(FH, "> %s\n", VEGAHdr);
    for(Atm = InizAtm; (Atm -> Ptr) && (Ret); Atm = Atm -> Ptr)
      if ((PrecRes != Atm -> ResName.L) || (PrecNum != Atm -> ResSeq.L)) {
        PrecRes = Atm -> ResName.L;
        PrecNum = Atm -> ResSeq.L;
        switch(Type) {
        case MOL_PROT:
          AATab1 = AATab;
          for(AATab3 = AAResTab; *AATab3 -> C; ++AATab3) {
            if (AATab3 -> L == PrecRes) {
              if (fputc(*AATab1, FH) == *AATab1) ++TotRes;
              else Ret = FALSE;
            }
            ++AATab1;
          }
          break;

        case MOL_NUCAC:
          if ((PrecRes == 0x41000000) || (PrecRes == 0x54000000) ||
              (PrecRes == 0x43000000) || (PrecRes == 0x47000000) ||
              (PrecRes == 0x55000000)) {
            if (fputc(*Atm -> ResName.C, FH) == *Atm -> ResName.C) ++TotRes;
            else Ret = FALSE;
          }
          break;
        }
        if ((!(TotRes % 10)) && (fputc(' ', FH) != ' ')) Ret = FALSE;
        if ((!(TotRes % 70)) && (fputc('\n', FH) != '\n')) Ret = FALSE;
      }
    if ((TotRes % 70) && (fputc('\n', FH) != '\n')) Ret = FALSE;
  } else {
    Errore("Input file is not protein or DNA/RNA");
    Ret = FALSE;
  }

  return Ret;
}


/**** Search the type of molecule ****/

UWORD TypeOfMol(register ATOMO *Atm)
{
  register UWORD	k;
  UWORD			Type;

  register LONG		PrecRes = 0;
  ULONG			TotAA   = 0;
  ULONG			TotNuc  = 0;
  ULONG			TotRes  = 0;

  while(Atm -> Ptr) {
    if (PrecRes != Atm -> ResName.L) {
      PrecRes = Atm -> ResName.L;
      if ((PrecRes == 0x41000000) || (PrecRes == 0x54000000) ||
          (PrecRes == 0x43000000) || (PrecRes == 0x47000000) ||
          (PrecRes == 0x55000000)) ++TotNuc;
      else {
        for(k = 0; k < 20; ++k)
          if (AAResTab[k].L == PrecRes) {
            ++TotAA;
            break;
          }
      }
      ++TotRes;
    }
    Atm = Atm -> Ptr;
  }
  TotRes *= TYPETOL;

  if ((TotAA) && (TotAA >= TotRes)) Type = MOL_PROT;
  else if ((TotNuc) && (TotNuc >= TotRes)) Type = MOL_NUCAC;

  return Type;
}
